@@ -1,7 +1,7 @@
 /** @file weight.h
  * @brief Weighting scheme API.
  */
-/* Copyright (C) 2004,2007,2008,2009,2010,2011,2012,2015,2016 Olly Betts
+/* Copyright (C) 2004,2007,2008,2009,2010,2011,2012,2015,2016,2019 Olly Betts
  * Copyright (C) 2009 Lemur Consulting Ltd
  * Copyright (C) 2013,2014 Aarsh Shah
  * Copyright (C) 2016 Vivek Pal
@@ -63,7 +63,12 @@ class XAPIAN_VISIBILITY_DEFAULT Weight {
 	/// Sum of wdf over the whole collection for the current term.
 	COLLECTION_FREQ = 4096,
 	/// Number of unique terms in the current document.
-	UNIQUE_TERMS = 8192
+	UNIQUE_TERMS = 8192,
+	/** Sum of lengths of all documents in the collection.
+	 *
+	 *  This gives the total number of term occurrences.
+	 */
+	TOTAL_LENGTH = COLLECTION_SIZE | AVERAGE_LENGTH
     } stat_flags;
 
     /** Tell Xapian that your subclass will want a particular statistic.
@@ -315,6 +320,15 @@ class XAPIAN_VISIBILITY_DEFAULT Weight {
 	return stats_needed & UNIQUE_TERMS;
     }
 
+    /// @private @internal Test if this is a BoolWeight object.
+    bool is_bool_weight_() const {
+	// Checking the name isn't ideal, but (get_maxpart() == 0.0) isn't
+	// required to work without init() having been called.  We can at
+	// least avoid the virtual method call in most non-BoolWeight cases
+	// as most other classes will need at least some stats.
+	return stats_needed == 0 && name() == "Xapian::BoolWeight";
+    }
+
   protected:
     /** Don't allow copying.
      *
@@ -372,6 +386,11 @@ class XAPIAN_VISIBILITY_DEFAULT Weight {
     Xapian::termcount get_wdf_upper_bound() const {
 	return wdf_upper_bound_;
     }
+
+    /// Total length of all documents in the collection.
+    Xapian::totallength get_total_length() const {
+	return Xapian::totallength(average_length_ * collection_size_ + 0.5);
+    }
 };
 
 /** Class implementing a "boolean" weighting scheme.
@@ -812,7 +831,7 @@ class XAPIAN_VISIBILITY_DEFAULT InL2Weight : public Weight {
   public:
     /** Construct an InL2Weight.
      *
-     *  @param c  A non-negative and non zero parameter controlling the extent
+     *  @param c  A strictly positive parameter controlling the extent
      *		  of the normalization of the wdf to the document length. The
      *		  default value of 1 is suitable for longer queries but it may
      *		  need to be changed for shorter queries. For more information,
@@ -884,7 +903,7 @@ class XAPIAN_VISIBILITY_DEFAULT IfB2Weight : public Weight {
   public:
     /** Construct an IfB2Weight.
      *
-     *  @param c  A non-negative and non zero parameter controlling the extent
+     *  @param c  A strictly positive parameter controlling the extent
      *		  of the normalization of the wdf to the document length. The
      *		  default value of 1 is suitable for longer queries but it may
      *		  need to be changed for shorter queries. For more information,
@@ -957,7 +976,7 @@ class XAPIAN_VISIBILITY_DEFAULT IneB2Weight : public Weight {
   public:
     /** Construct an IneB2Weight.
      *
-     *  @param c  A non-negative and non zero parameter controlling the extent
+     *  @param c  A strictly positive parameter controlling the extent
      *		  of the normalization of the wdf to the document length. The
      *		  default value of 1 is suitable for longer queries but it may
      *		  need to be changed for shorter queries. For more information,
@@ -1031,7 +1050,7 @@ class XAPIAN_VISIBILITY_DEFAULT BB2Weight : public Weight {
   public:
     /** Construct a BB2Weight.
      *
-     *  @param c  A non-negative and non zero parameter controlling the extent
+     *  @param c  A strictly positive parameter controlling the extent
      *		  of the normalization of the wdf to the document length. A
      *		  default value of 1 is suitable for longer queries but it may
      *		  need to be changed for shorter queries. For more information,
@@ -1103,15 +1122,14 @@ class XAPIAN_VISIBILITY_DEFAULT DLHWeight : public Weight {
 
   public:
     DLHWeight() {
-	need_stat(AVERAGE_LENGTH);
 	need_stat(DOC_LENGTH);
-	need_stat(COLLECTION_SIZE);
 	need_stat(COLLECTION_FREQ);
 	need_stat(WDF);
 	need_stat(WQF);
 	need_stat(WDF_MAX);
 	need_stat(DOC_LENGTH_MIN);
 	need_stat(DOC_LENGTH_MAX);
+	need_stat(TOTAL_LENGTH);
     }
 
     std::string name() const;
@@ -1176,7 +1194,7 @@ class XAPIAN_VISIBILITY_DEFAULT PL2Weight : public Weight {
   public:
     /** Construct a PL2Weight.
      *
-     *  @param c  A non-negative and non zero parameter controlling the extent
+     *  @param c  A strictly positive parameter controlling the extent
      *		  of the normalization of the wdf to the document length. The
      *		  default value of 1 is suitable for longer queries but it may
      *		  need to be changed for shorter queries. For more information,
@@ -1246,7 +1264,7 @@ class XAPIAN_VISIBILITY_DEFAULT PL2PlusWeight : public Weight {
   public:
     /** Construct a PL2PlusWeight.
      *
-     *  @param c  A non-negative and non zero parameter controlling the extent
+     *  @param c  A strictly positive parameter controlling the extent
      *		  of the normalization of the wdf to the document length. The
      *		  default value of 1 is suitable for longer queries but it may
      *		  need to be changed for shorter queries. For more information,
@@ -1328,15 +1346,14 @@ class XAPIAN_VISIBILITY_DEFAULT DPHWeight : public Weight {
   public:
     /** Construct a DPHWeight. */
     DPHWeight() {
-	need_stat(AVERAGE_LENGTH);
 	need_stat(DOC_LENGTH);
-	need_stat(COLLECTION_SIZE);
 	need_stat(COLLECTION_FREQ);
 	need_stat(WDF);
 	need_stat(WQF);
 	need_stat(WDF_MAX);
 	need_stat(DOC_LENGTH_MIN);
 	need_stat(DOC_LENGTH_MAX);
+	need_stat(TOTAL_LENGTH);
     }
 
     std::string name() const;
@@ -1433,9 +1450,7 @@ class XAPIAN_VISIBILITY_DEFAULT LMWeight : public Weight {
 	    else
 		param_smoothing2 = 0.05;
 	}
-	need_stat(AVERAGE_LENGTH);
 	need_stat(DOC_LENGTH);
-	need_stat(COLLECTION_SIZE);
 	need_stat(RSET_SIZE);
 	need_stat(TERMFREQ);
 	need_stat(RELTERMFREQ);
@@ -1443,6 +1458,7 @@ class XAPIAN_VISIBILITY_DEFAULT LMWeight : public Weight {
 	need_stat(WDF);
 	need_stat(WDF_MAX);
 	need_stat(COLLECTION_FREQ);
+	need_stat(TOTAL_LENGTH);
 	if (select_smoothing == ABSOLUTE_DISCOUNT_SMOOTHING)
 	    need_stat(UNIQUE_TERMS);
 	if (select_smoothing == DIRICHLET_PLUS_SMOOTHING)
@@ -1452,7 +1468,7 @@ class XAPIAN_VISIBILITY_DEFAULT LMWeight : public Weight {
     std::string name() const;
 
     std::string serialise() const;
-    LMWeight * unserialise(const std::string & s) const;
+    LMWeight * unserialise(const std::string & serialised) const;
 
     double get_sumpart(Xapian::termcount wdf,
 		       Xapian::termcount doclen,
@@ -1473,7 +1489,7 @@ class XAPIAN_VISIBILITY_DEFAULT CoordWeight : public Weight {
     double factor;
 
   public:
-    CoordWeight * clone() const ;
+    CoordWeight * clone() const;
 
     void init(double factor_);
 
@@ -1483,7 +1499,7 @@ class XAPIAN_VISIBILITY_DEFAULT CoordWeight : public Weight {
     std::string name() const;
 
     std::string serialise() const;
-    CoordWeight * unserialise(const std::string &) const;
+    CoordWeight * unserialise(const std::string & serialised) const;
 
     double get_sumpart(Xapian::termcount wdf,
 		       Xapian::termcount doclen,
